# -*- coding: utf-8 -*-
# Copyright 2020 Aneior Studio, SL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Carte hook module"""


import requests
import xmltodict
from urllib.parse import urlencode

from airflow import AirflowException
from airflow.hooks.base import BaseHook
from requests.auth import HTTPBasicAuth


class PentahoCarteHook(BaseHook):
    """Implementation hook for interact with Carte Rest API"""

    class PentahoCarteClient:
        """Implementation for Carte calls"""

        RUN_JOB_ENDPOINT = '/kettle/executeJob/'
        JOB_STATUS_ENDPOINT = '/kettle/jobStatus/'
        RUN_TRANS_ENDPOINT = '/kettle/executeTrans/'
        TRANS_STATUS_ENDPOINT = '/kettle/transStatus/'

        def __init__(
                self,
                host,
                port,
                rep,
                username,
                password,
                carte_username,
                carte_password,
                *args,
                level='Basic',
                **kwargs):
            super().__init__(*args, **kwargs)
            self.host = host
            if not self.host.startswith('http'):
                self.host = 'http://{}'.format(self.host)
            self.port = port
            self.rep = rep
            self.username = username
            self.password = password
            self.carte_username = carte_username
            self.carte_password = carte_password
            self.level = level

        def __get_url(self, endpoint):
            return '{}:{}{}'.format(self.host, self.port, endpoint)

        def __get_auth(self):
            return HTTPBasicAuth(self.carte_username, self.carte_password)

        def job_status(self, job_name, job_id, previous_response=None):
            url = self.__get_url(self.JOB_STATUS_ENDPOINT)
            headers = {'Content-Type': 'application/x-www-form-urlencoded'}

            from_line = previous_response['jobstatus']['last_log_line_nr'] \
                if previous_response \
                else 0

            payload = {
                'name': job_name,
                'id': job_id,
                'xml': 'Y',
                'from': from_line
            }

            rs = requests.post(url=url, headers=headers,
                               data=urlencode(payload), auth=self.__get_auth())
            if rs.status_code >= 400:
                result = xmltodict.parse(rs.content)
                raise AirflowException('{}: {}'.format(
                    result['webresult']['result'],
                    result['webresult']['message'])
                )
            else:
                return xmltodict.parse(rs.content)

        def run_job(self, job_path, params=None):
            url = self.__get_url(self.RUN_JOB_ENDPOINT)
            args = {
                'user': self.username,
                'pass': self.password,
                'rep': self.rep,
                'job': job_path,
                'level': 'Debug'
            }

            if params:
                args.update(params)

            rs = requests.get(url=url, params=args, auth=self.__get_auth())
            if rs.status_code >= 400:
                result = xmltodict.parse(rs.content)
                raise AirflowException('{}: {}'.format(
                    result['webresult']['result'],
                    result['webresult']['message'])
                )
            else:
                return xmltodict.parse(rs.content)

        def trans_status(self, trans_name, previous_response=None):
            url = self.__get_url(self.TRANS_STATUS_ENDPOINT)
            headers = {'Content-Type': 'application/x-www-form-urlencoded'}

            from_line = previous_response['transstatus']['last_log_line_nr'] \
                if previous_response \
                else 0

            payload = {
                'name': trans_name,
                'xml': 'Y',
                'from': from_line
            }

            rs = requests.post(url=url, headers=headers,
                               data=urlencode(payload), auth=self.__get_auth())
            if rs.status_code >= 400:
                result = xmltodict.parse(rs.content)
                raise AirflowException('{}: {}'.format(
                    result['webresult']['result'],
                    result['webresult']['message'])
                )
            else:
                return xmltodict.parse(rs.content)

        def run_trans(self, trans_path, params=None):
            url = self.__get_url(self.RUN_TRANS_ENDPOINT)
            args = {
                'user': self.username,
                'pass': self.password,
                'rep': self.rep,
                'trans': trans_path,
                'level': 'Debug'
            }

            if params:
                args.update(params)

            rs = requests.get(url=url, params=args, auth=self.__get_auth())
            if rs.status_code >= 400:
                raise AirflowException(rs.content)

    def __init__(self, *args, conn_id='pdi_default', level='Basic', **kwargs):
        super().__init__(*args, **kwargs)
        self.conn_id = conn_id
        self.level = level
        self.connection = self.get_connection(conn_id)
        self.extras = self.connection.extra_dejson
        self.pentaho_cli = None

    def get_conn(self):
        """
        Provide required object to run jobs on Carte
        :return:
        """
        if self.pentaho_cli:
            return self.pentaho_cli

        self.pentaho_cli = self.PentahoCarteClient(
            host=self.connection.host,
            port=self.connection.port,
            rep=self.extras.get('rep'),
            username=self.connection.login,
            password=self.connection.password,
            carte_username=self.extras.get('carte_username'),
            carte_password=self.extras.get('carte_password'),
            level=self.level)

        return self.pentaho_cli



# -*- coding: utf-8 -*-
# Copyright 2020 Aneior Studio, SL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Carte Base Operator module"""


import base64
import re
import time
import zlib

from airflow import AirflowException
from airflow.models import BaseOperator

from airflow_pentaho.hooks.carte import PentahoCarteHook


class CarteBaseOperator(BaseOperator):
    """Carte Base Operator"""

    FINISHED_STATUSES = ['Finished', 'Stopped']

    DEFAULT_CONN_ID = 'pdi_default'

    template_fields = ('params',)

    def _log_logging_string(self, raw_logging_string):
        logs = raw_logging_string
        cdata = re.match(r'\<\!\[CDATA\[([^\]]+)\]\]\>', logs)
        cdata = cdata.group(1) if cdata else raw_logging_string
        decoded_lines = zlib.decompress(base64.b64decode(cdata),
                                        16 + zlib.MAX_WBITS)
        if decoded_lines:
            for line in re.compile(r'\r\n|\n|\r').split(
                    decoded_lines.decode('utf-8')):
                self.log.info(line)


class CarteJobOperator(CarteBaseOperator):
    """Carte Job operator. Runs job on Carte service."""

    LOG_TEMPLATE = '%s: %s, with id %s'

    def __init__(self,
                 job,
                 *args,
                 params=None,
                 pdi_conn_id=None,
                 level='Basic',
                 **kwargs):
        """
        Execute a Job in a remote Carte server from a PDI repository.
        :param job: The full path of the job
        :type job: str
        :param params: Set a named parameter in a dict as input parameters.
        :type params: dict
        :param pdi_conn_id: Pentaho Data Integration connection ID.
        :type pdi_conn_id: str
        :param level: The logging level (Basic, Detailed, Debug, Rowlevel,
            Error, Nothing), default is Basic.
        :type level: str
        """
        super().__init__(*args, **kwargs)

        self.pdi_conn_id = pdi_conn_id
        if not self.pdi_conn_id:
            self.pdi_conn_id = self.DEFAULT_CONN_ID
        self.job = job
        self.level = level
        self.params = params

    def _get_pentaho_carte_client(self):
        return PentahoCarteHook(conn_id=self.pdi_conn_id,
                                level=self.level).get_conn()

    def _get_job_name(self):
        return self.job.split('/').pop()

    def execute(self, context):  # pylint: disable=unused-argument
        conn = self._get_pentaho_carte_client()

        exec_job_rs = conn.run_job(self.job, self.params)
        message = exec_job_rs['webresult']['message']
        job_id = exec_job_rs['webresult']['id']
        self.log.info('%s: %s, with id %s', message, self.job, job_id)

        status_job_rs = None
        status = None
        status_desc = None
        while not status_job_rs or status_desc not in self.FINISHED_STATUSES:
            status_job_rs = conn.job_status(self._get_job_name(), job_id,
                                            status_job_rs)
            status = status_job_rs['jobstatus']
            status_desc = status['status_desc']
            self.log.info(self.LOG_TEMPLATE, status_desc, self.job, job_id)
            self._log_logging_string(status['logging_string'])

            if status_desc not in self.FINISHED_STATUSES:
                self.log.info('Sleeping 5 seconds before ask again')
                time.sleep(5)

        if 'error_desc' in status and status['error_desc']:
            self.log.error(self.LOG_TEMPLATE, status['error_desc'],
                           self.job, job_id)
            raise AirflowException(status['error_desc'])


class CarteTransOperator(CarteBaseOperator):
    """Cart Transformation operator. Runs job on Carte service."""

    LOG_TEMPLATE = '%s: %s'

    def __init__(self,
                 trans,
                 *args,
                 params=None,
                 pdi_conn_id=None,
                 level='Basic',
                 **kwargs):
        """
        Execute a Transformation in a remote Carte server from a PDI
            repository.
        :param trans: The full path of the transformation.
        :type trans: str
        :param params: Set a named parameter in a dict as input parameters.
        :type params: dict
        :param pdi_conn_id: Pentaho Data Integration connection ID.
        :type pdi_conn_id: str
        :param level: The logging level (Basic, Detailed, Debug, Rowlevel,
            Error, Nothing), default is Basic.
        :type level: str
        """
        super().__init__(*args, **kwargs)

        self.pdi_conn_id = pdi_conn_id
        if not self.pdi_conn_id:
            self.pdi_conn_id = self.DEFAULT_CONN_ID
        self.trans = trans
        self.level = level
        self.params = params

    def _get_pentaho_carte_client(self):
        return PentahoCarteHook(conn_id=self.pdi_conn_id,
                                level=self.level).get_conn()

    def _get_trans_name(self):
        return self.trans.split('/').pop()

    def execute(self, context):  # pylint: disable=unused-argument
        conn = self._get_pentaho_carte_client()

        conn.run_trans(self.trans, self.params)
        self.log.info('Executing {}'.format(self.trans))

        status_trans_rs = None
        status = None
        status_desc = None
        while not status_trans_rs or status_desc not in self.FINISHED_STATUSES:
            status_trans_rs = conn.trans_status(self._get_trans_name(),
                                                status_trans_rs)
            status = status_trans_rs['transstatus']
            status_desc = status['status_desc']
            self.log.info(self.LOG_TEMPLATE, status_desc, self.trans)
            self._log_logging_string(status['logging_string'])

            if status_desc not in self.FINISHED_STATUSES:
                self.log.info('Sleeping 5 seconds before ask again')
                time.sleep(5)

        if 'error_desc' in status and status['error_desc']:
            self.log.error(self.LOG_TEMPLATE, status['error_desc'], self.trans)
            raise AirflowException(status['error_desc'])



# -*- coding: utf-8 -*-
# pylint: disable=invalid-name
"""This module is deprecated. Please use `airflow_pentaho.operators.kitchen`."""

import warnings

# pylint: disable=unused-import
from airflow_pentaho.operators.kettle import KitchenOperator  # noqa

warnings.warn(
    'This module is deprecated. Please use `airflow_pentaho.operators.kitchen`.',
    DeprecationWarning,
    stacklevel=2)



# -*- coding: utf-8 -*-
# Copyright 2020 Aneior Studio, SL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Setup"""


from setuptools import find_packages, setup

from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='airflow-pentaho-plugin',
    version_format='{tag}.post{commitcount}',
    license='Apache 2.0',
    setup_requires=['setuptools-git-version'],
    author='Damavis',
    author_email='info@damavis.com',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/damavis/airflow-pentaho-plugin',
    python_requires='>=3.6',
    test_suite='nose.collector',
    zip_safe=False,
    include_package_data=True,
    packages=find_packages('.', exclude=['tests', 'tests.*']),
    classifiers=[
        'Environment :: Plugins',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: Unix',
        'Programming Language :: Python :: 3'
    ],
    test_requires=[
        'apache-airflow >= 2.0.0'
    ],
    install_requires=[
      'xmltodict >= 0.10.0',
    ],
    entry_points={
        'airflow.plugins': [
            'airflow_pentaho = airflow_pentaho.plugin:PentahoPlugin'
        ]
    }
)
